@@ -1,4 +1,4 @@
-# Copyright © 2017 Tom Hacohen
+# Copyright © 2017-2021 Tom Hacohen <tom@stosb.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -46,6 +46,7 @@ def needs_ssl():
     return (ON_MAC or ON_WINDOWS) and \
         not has_ssl()
 
+
 def generate_cert(cert_path: str = SSL_CERT_FILE, key_path: str = SSL_KEY_FILE,
                   key_size: int = KEY_SIZE, key_days: int = KEY_DAYS):
     if os.path.exists(key_path):
@@ -69,8 +70,8 @@ def generate_cert(cert_path: str = SSL_CERT_FILE, key_path: str = SSL_KEY_FILE,
 
     san = x509.SubjectAlternativeName(alt_names)
 
-    # path_len=0 means this cert can only sign itself, not other certs.
-    basic_contraints = x509.BasicConstraints(ca=True, path_length=0)
+    # prevent this cert form being used to sign other certs
+    basic_contraints = x509.BasicConstraints(ca=False, path_length=None)
     now = datetime.utcnow()
     cert = (
         x509.CertificateBuilder()
@@ -105,9 +106,19 @@ def macos_trust_cert(cert_path: str = SSL_CERT_FILE):
 
 
 def windows_trust_cert(cert_path: str = SSL_CERT_FILE):
+    """Import given certificate into a certificate store."""
     if not ON_WINDOWS:
         raise Error('this is not Windows.')
-    check_call(['powershell.exe', 'Import-Certificate', '-FilePath', '"{}"'.format(cert_path), '-CertStoreLocation', 'Cert:\CurrentUser\Root'])
+    check_call(
+        [
+            'powershell.exe',
+            'Import-Certificate',
+            '-FilePath',
+            '"{}"'.format(cert_path),
+            '-CertStoreLocation',
+            r'Cert:\CurrentUser\Root',
+        ]
+    )
 
 
 def trust_cert(cert_path: str = SSL_CERT_FILE):
