@@ -19,13 +19,10 @@ import threading
 import hashlib
 import posixpath
 import time
-from uuid import uuid4
 
-from .etesync_cache import EteSyncCache, etesync_for_user
+from .etesync_cache import etesync_for_user
 from .href_mapper import HrefMapper
 
-import etesync_dav.config as config
-
 import etesync as api
 from radicale import pathutils
 from radicale.item import Item, get_etag
@@ -34,6 +31,9 @@ from radicale.storage import (
     )
 import vobject
 
+from ..local_cache import Etebase, COL_TYPES
+from .storage_etebase_collection import Collection as EtebaseCollection
+
 
 logger = logging.getLogger('etesync-dav')
 
@@ -52,6 +52,7 @@ class SyncThread(threading.Thread):
         self._done_syncing.set()  # We are done before we start.
         self.user = user
         self.last_sync = None
+        self._exception = None
 
     def force_sync(self):
         self._force_sync.set()
@@ -66,7 +67,12 @@ class SyncThread(threading.Thread):
         return self._force_sync.is_set()
 
     def wait_for_sync(self, timeout=None):
-        return self._done_syncing.wait(timeout)
+        ret = self._done_syncing.wait(timeout)
+        e = self._exception
+        self._exception = None
+        if e is not None:
+            raise e
+        return ret
 
     def run(self):
         while True:
@@ -79,6 +85,7 @@ class SyncThread(threading.Thread):
             except Exception as e:
                 # Print errors but keep on syncing in the background
                 logger.exception(e)
+                self._exception = e
             finally:
                 self._force_sync.clear()
                 self._done_syncing.set()
@@ -86,7 +93,6 @@ class SyncThread(threading.Thread):
             self._force_sync.wait(SYNC_INTERVAL)
 
 
-
 class MetaMapping:
     # Mappings between etesync meta and radicale
     _mappings = {
@@ -297,7 +303,7 @@ class Collection(BaseCollection):
         ValueError is raised for invalid or old tokens.
         """
         token_prefix = 'http://radicale.org/ns/sync/'
-        token = "{}{}".format(token_prefix, self.etag.strip('"'))
+        token = None  # XXX "{}{}".format(token_prefix, self.etag.strip('"'))
         if old_token is not None and old_token.startswith(token_prefix):
             old_token = old_token[len(token_prefix):]
 
@@ -377,12 +383,16 @@ class Collection(BaseCollection):
                             del item.contents['photo']
 
                     item.contents['version'][0].value = '3.0'
+            # XXX Hack: add missing FN
+            if item.name == 'VCARD' and not hasattr(item, 'fn'):
+                item.add('fn').value = str(item.n)
         except Exception as e:
             raise RuntimeError("Failed to parse item %r in %r" %
                                (href, self.path)) from e
         last_modified = ''
 
-        return EteSyncItem(collection=self, vobject_item=item, href=href, last_modified=last_modified, etesync_item=etesync_item)
+        return EteSyncItem(collection=self, vobject_item=item, href=href, last_modified=last_modified,
+                           etesync_item=etesync_item)
 
     def upload(self, href, vobject_item):
         """Upload a new or replace an existing item."""
@@ -474,13 +484,14 @@ class Collection(BaseCollection):
 class Storage(BaseStorage):
     """Collection stored in several files per calendar."""
 
-    _collection_class = Collection
-
     _sync_thread_lock = threading.RLock()
+    # Per-object lock for the "global" user and etesync
+    _etesync_user_lock = None
 
     def __init__(self, configuration):
         self.user = None
         self.etesync = None
+        self._etesync_user_lock = threading.RLock()
         super().__init__(configuration)
 
     def discover(self, path, depth="0"):
@@ -498,7 +509,10 @@ class Storage(BaseStorage):
 
         """
 
-        cls = self._collection_class
+        if isinstance(self.etesync, Etebase):
+            cls = EtebaseCollection
+        else:
+            cls = Collection
 
         # Path should already be sanitized
         attributes = _get_attributes_from_path(path)
@@ -533,9 +547,14 @@ class Storage(BaseStorage):
         if len(attributes) == 0:
             yield cls(self, posixpath.join(path, cls.user))
         elif len(attributes) == 1:
-            for journal in self.etesync.list():
-                if journal.collection.TYPE in (api.AddressBook.TYPE, api.Calendar.TYPE, api.TaskList.TYPE):
-                    yield cls(self, posixpath.join(path, journal.uid))
+            if isinstance(self.etesync, Etebase):
+                for journal in self.etesync.list():
+                    if journal.col_type in COL_TYPES:
+                        yield cls(self, posixpath.join(path, journal.uid))
+            else:
+                for journal in self.etesync.list():
+                    if journal.collection.TYPE in (api.AddressBook.TYPE, api.Calendar.TYPE, api.TaskList.TYPE):
+                        yield cls(self, posixpath.join(path, journal.uid))
         elif len(attributes) == 2:
             for href in collection._list():
                 yield collection._get(href)
@@ -602,7 +621,7 @@ class Storage(BaseStorage):
         # At most wait for 5 seconds before returning stale data
         etesync.sync_thread.wait_for_sync(5)
 
-        with etesync_for_user(user) as (etesync, _):
+        with self._etesync_user_lock, etesync_for_user(user) as (etesync, _):
             self.user = user
             self.etesync = etesync
 
